package vio.tracker;

import vio.tracker.*;
import vio.tracker.io.*;

import javax.comm.*;
import java.io.*;
import java.net.*;

public class TrackerManager {

	private TrackerData myTrackerData;
	private TrackerInfo myTrackerInfo;

	private boolean verbose = false;
	private boolean status  = true;
	
	private byte[] binaryTrackerData = new byte[8];
	private byte[] emptyTrackerData;
	private String asciiTrackerData;

	private SerialPort trackerPort;
	private String initString;

	private DataInputStream in;
	private PrintStream out;

	private CommPortIdentifier trackerPortIdentifier;

	private int magnetic_filter;
	private int tilt_filter;

	private TrackerInputStream tr_in;
	private TrackerOutputStream tr_out;
	public TrackerReader tr_rd;

	// constructor without aguments
	// searches automatically for the tracker
	// and uses default settings
	public TrackerManager(boolean verbose) {

		this.verbose = verbose;
		
		try { trackerPortIdentifier = TrackerScanner.findTracker(verbose); }
		catch (TrackerNotFoundException e) {
			if (verbose)
				System.out.println("no tracker found");
		}

		initString = "M2,C,A,3,3";
	}
		

	// constuctor with the name of the trackerPort
	// uses default settings
	public TrackerManager(String portName, boolean verbose) {

		this.verbose = verbose;

		try {
			trackerPortIdentifier = CommPortIdentifier.getPortIdentifier(portName);
		}
		catch ( NoSuchPortException e) { System.out.println ("no port "+portName); }

		initString = "M2,C,A,5,5";
	}

	// constuctor with the name of the trackerPort
	// personal filter settings
	public TrackerManager(String portName, int magnetic_filter, int tilt_filter, boolean verbose) {
	
		this.verbose = verbose;

		try {
			trackerPortIdentifier = CommPortIdentifier.getPortIdentifier(portName);
		}
		catch ( NoSuchPortException e) { System.out.println ("no port "+portName); }

		this.magnetic_filter = magnetic_filter;
		this.tilt_filter = tilt_filter;

		initString = "M2,C,A,"+magnetic_filter+","+tilt_filter;
	}

	// constuctor with a CommPortIdentifier
	// uses default settings
	public TrackerManager(CommPortIdentifier trackerPortIdentifier, boolean verbose) {

		this.verbose = verbose;
	
		this.trackerPortIdentifier = trackerPortIdentifier;

		initString = "M2,C,A,3,3";
	}

	// constuctor with a CommPortIdentifier
	// personal filter settings
	public TrackerManager(CommPortIdentifier trackerPortIdentifier, int magnetic_filter, int tilt_filter, boolean verbose) {
	
		this.verbose = verbose;

		this.trackerPortIdentifier = trackerPortIdentifier;

		this.magnetic_filter = magnetic_filter;
		this.tilt_filter = tilt_filter;

		initString = "M2,C,A,"+magnetic_filter+","+tilt_filter;
	}
	

	public static String getVersion() {
		return "0.82";
	}

	// opens the serialPort & initializes the tracker
	public boolean init() {
	
		myTrackerData = new TrackerData();

		//opens the SerialPort
		try {
			trackerPort = (SerialPort)trackerPortIdentifier.open("vio.tracker.TrackerManager",1000);
			if (verbose)
				System.out.println("\nopened SerialPort "+trackerPortIdentifier.getName());

		}
		catch (PortInUseException e) {
			if (verbose)
				System.out.println("SerialPort "+trackerPortIdentifier.getName()+" is already in use by "+trackerPortIdentifier.getCurrentOwner() );
			return false;
		}
		catch (NullPointerException e) {
			return false;
		}

		// configures the serialport
		try { trackerPort.setSerialPortParams(19200,8,1,SerialPort.PARITY_NONE); }
		catch (UnsupportedCommOperationException e) { System.out.println(e); }

		//initializes the tracker
		try {
			in =  new DataInputStream(trackerPort.getInputStream());
			out = new PrintStream(trackerPort.getOutputStream());

			tr_in = new TrackerInputStream(in);
			tr_out = new TrackerOutputStream(out);
			
			//tr_rd = new TrackerReader(in,out);

		}
		catch (IOException e) { return false; }

		return ( reset() );
	}


	public boolean reset() {
	
		try {
			try { tr_rd.stop(); tr_rd=null; }
			catch (NullPointerException e) {}
			
			tr_rd = new TrackerReader(in,out);

			int tries=0;
			// resets the tracker to the factory defaults and 
			// sets the communication parameters
			if (verbose)
				System.out.print("resetting the tracker");
			while ( (!tr_in.checkAnswer()) && (tries<100) ) {
				tr_out.sendCommand("R");
					try { Thread.sleep(10);
					} catch (InterruptedException e) {}
					if (verbose)
						System.out.print(".");	
					tries++;	
			}
			
			if (tries==100) {
				if (verbose)
					System.out.println("\ntracker not responding");
				trackerPort.close();
				return false;
			}
			else {
				if (verbose)
					System.out.println(" OK");
				tries=0;
			}

			// initializes the tracker with the parameters 
			// as set in the constructor
			if (verbose)
				System.out.print("initializing the tracker");
			while ( (!tr_in.checkAnswer()) && (tries<100) ) {
				tr_out.sendCommand(initString);
				try { Thread.sleep(10);
				} catch (InterruptedException e) {}
				if (verbose)
					System.out.print(".");
				tries++;
			}

			if (tries==100) {
				if (verbose)
					System.out.println("\ntracker not responding");
				trackerPort.close();
				return false;
			}
			else {
				if (verbose)
					System.out.println(" OK");
				tries=0;
			}
			
			//gets the TrackerInfo
			requestTrackerInfo();
			tr_rd.start();

		}
		catch (Exception e) { return false; }

		return true;
	}
	
		
	public TrackerData getTrackerData() throws TrackerManagerException {
				
		if (!tr_rd.running) {
			status=false;
			throw new TrackerManagerException();
		}
		else {
			try {
				asciiTrackerData = tr_rd.getData();
				myTrackerData=new TrackerData( asciiTrackerData );
			}
			catch (Exception e) { System.out.println(e);}
		}
		return myTrackerData;
	}
	
	
	public String getTrackerPortName() {

		return trackerPortIdentifier.getName();
	}
	
	public TrackerInfo getTrackerInfo() {

		return myTrackerInfo;	
	}

	public boolean getStatus() {
	
		return status;	
	}
	
	private void requestTrackerInfo() {
		tr_in.readGarbage();	
		try {
			tr_out.sendCommand("V");
			asciiTrackerData = tr_in.readVersion();
			myTrackerInfo = new TrackerInfo(asciiTrackerData);
		}
		catch (Exception e) {}
		tr_in.readGarbage();
	}

	//closes the tracker
	public void close() {
		out.print("!");
		tr_rd.stop();
		tr_rd=null;
		trackerPort.close();
	}

}
